#-------------------------------------------------------------------------------
# Name:        Thermostat Demo AWS IoT
# Purpose:     To illistrait how to use the AT commands to
#              implement a basic thermostat using AWS Iot Cloud Services.
#
# Author:      sep
#
# Created:     12/10/2015
# Copyright:   (c) Inventek Systems, LLC. 2015
#
# Name:        INVENTEK WIRELESS INTEROPERABILITY NETWORK (IWIN), AT Command Software
# Purpose:     To provide Inventek customers Python scripts illustrating how to
#              use the IWIN AT Command Software.
#
#------------------------------------------------------------------------------
#LICENSE:
#   THE IWIN PROGRAM IS FREE SOFTWARE FOR INVENTEK CUSTOMERS ONLY.
#   INVENTEK SYSTEMS RETAINS ALL OWNERSHIP AND INTELLECTUAL PROPERTY RIGHTS
#   IN THE IWIN CODE ACCOMPANYING THIS MESSAGE AND IN ALL DERIVATIVES HERETO.
#   USER MAY USE THE IWIN CODE, AND ANY DERIVATIVES CREATED BY ANY PERSON OR
#   ENTITY BY OR ON USER?S BEHALF, EXCLUSIVELY WITH INVENTEK'S PROPRIETARY
#   PRODUCTS.
#
#   USER AGREES THAT USER IS SOLELY RESPONSIBLE FOR TESTING THE CODE AND
#   DETERMINING IT?S SUITABILITY.  INVENTEK HAS NO OBLIGATION TO MODIFY, TEST,
#   CERTIFY, OR SUPPORT THE CODE.
#
#   USER ACCEPTANCE AND/OR USE OF THE IWIN CODE CONSTITUTES AGREEMENT TO THE
#   TERMS AND CONDITIONS OF THIS NOTICE. IWIN CAN BE REDISTRIBUTED AND/OR
#   MODIFIED BY INVENTEK CUSTOMERS ONLY AND UNDER THE TERMS OF THE GNU GENERAL
#   PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION, EITHER VERSION
#   3 OF THE LICENSE, OR (AT USER?S OPTION) ANY LATER VERSION.
#
#   THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT
#   ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS
#   FOR A PARTICULAR PURPOSE.  REFERENCE THE GNU GENERAL PUBLIC LICENSE FOR MORE
#   DETAILS.
#
#   CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY INVENTEK "AS IS".  NO WARRANTIES,
#   WHETHER EXPRESSED, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
#   IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
#   PARTICULAR PURPOSE APPLY TO THIS CODE, IT?S INTERACTION WITH INVENTEK 'S
#   PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
#
#   USER ACKNOWLEDGES AND AGREES THAT, IN NO EVENT, SHALL INVENTEK BE LIABLE,
#   WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF
#   STATUTORY DUTY), STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE,
#   FOR ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL
#   LOSS, DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,
#   HOWSOEVER CAUSED, EVEN IF INVENTEK HAS BEEN ADVISED OF THE POSSIBILITY OR THE
#   DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW, INVENTEK 'S
#   TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS CODE, SHALL NOT
#   EXCEED THE PRICE PAID DIRECTLY TO INVENTEK SPECIFICALLY TO HAVE THIS CODE
#   DEVELOPED.
#
#   USER SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE ALONG
#   WITH THIS PROGRAM. IF NOT, SEE <HTTP://WWW.GNU.ORG/LICENSES/>.
#-------------------------------------------------------------------------------
#
# Version:      0.0         sep         20160310    Intial release
#-------------------------------------------------------------------------------

import math, sys, serial, serial.tools.list_ports, time, json

#Quiet mode, 0 = Quiet, 1 = Verbose
quiet = 0

#Send timeout
timeout = 30

#Show Hex data, 0 = Don't show, 1 = Show
showhex = 0

#Send a command to a eS-WiFi module and wait for prompt("\r\n> ") responce
def sendCMDBlocking(serPort,cmd):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(cmd)
    serPort.write(cmd + '\r')

    st = time.time()                    #Get time

    #read until "\r\n> " received
    found = 0
    while found != 1:
        for c in serPort.read():
            rcvString += c
            if  '\r\n> ' in rcvString:
                found = 1
            #Endif
        #End for

        #Check for timeout
        if (time.time() - st) > timeout:
            return '-1'                 #Timeout error
        #Endif
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString


#Send CMD and wait for responce, waiting nsec seconds before timing out
def sendCMDWait(serPort,cmd,responce,nsec):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(cmd)
    serPort.write(cmd + '\r')

    st = time.time()                    #Get time

    #Check each message recieved for responce
    found = 0
    while found != 1:

        #read until "\r\n> " received
        found0 = 0
        while found0 != 1:
            for c in serPort.read():
                rcvString += c
                #if (quiet == 1):
                sys.stdout.write(c)
                if  "\r\n> " in rcvString:
                    found0 = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > nsec:
                return '-1'                 #Timeout error
            #Endif
        #End while

        #Show responce
        if (quiet == 1):
            sys.stdout.write(rcvString)

        #Check for responce
        if  responce in rcvString:
            found = 1
        else:
            rcvString = ""                  #Clear string
        #End if
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString


#Send data to a eS-WiFi module and wait for prompt("\r\n> ") responce
#   Using a byte array with the S3 command embedded in array
#   e.g. S3=100\r<100 bytes of data>
def sendDataBlocking(serPort,data):
    rcvString =""

    #Send Command
    if (quiet == 1):
        #sys.stdout.write(data)
        if ('S0' in data):
            print(str(data[0:3]))
            if (showhex == 1): print binascii.hexlify(data[4:len(data)])
        #End if
        if ('S3' in data):
            tmp = str(data[:8])
            end = tmp.find('\r')
            print(data[:end])
            if (showhex ==1): print binascii.hexlify(data[end+1:len(data)])
        #end if
    #End if
    serPort.write(data)

    st = time.time()                    #Get time

    #read until "\r\n> " received
    found = 0
    while found != 1:
        for c in serPort.read():
            rcvString += c
            if  '\r\n> ' in rcvString:
                found = 1
            #Endif
        #End for

        #Check for timeout
        if (time.time() - st) > timeout:
            return '-1'                 #Timeout error
        #Endif
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString[2:])

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString


#Search COM port for and eS-WiFi module
def findModule(timeout):
    rcvString =""

    #Get ports
    portList = list(serial.tools.list_ports.comports())
    #print(portList)

    #Search for module
    for index, openPort in enumerate(portList):

        #Serial Port Setup
        port = portList[index][0]
        baud = 115200
        serPort = serial.Serial(port,baud)
        serPort.timeout = 10

        serPort.write('\r')                 #Send CR

        st = time.time()                    #Get time

        #read until "\r\n> " received
        found = 0
        while found != 1:
            for c in serPort.read():
                rcvString += c
                if  '\r\n> ' in rcvString:
                    found = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > timeout:
                return '-1'                 #Timeout error
            #Endif
        #End while

        #Try to get info
        rcv = sendCMDBlocking(serPort,"I?")

        #Close serial port
        serPort.close()

        #Found?
        if (found == 1):
            #print("Port = " + port)
            if "ISM4" in rcv:
                print(rcv)
                return port
            #Endif
        #End if
    #End for
    return -1

#Application
def main():

    #Redirect std err to std out
    sys.stderr = sys.stdout

    #Serial Port Setup
    port = findModule(10)
    if (port == -1):
        print( "Failed to find an eS-WiFi module!")
        sys.exit(-1)
    #End if
    baud = 115200
    ser = serial.Serial(port,baud)
    ser.timeout = 10

    #Get Firmware information
    if (quiet == 1):
        rcv = sendCMDBlocking(ser,"I?")
        print(rcv)
    #End if

    #Setup----------------------------------------------------------------------
    #Network Configuration Method
    useSoftAP = 0               #Join using, 0 = Cx commands, 1 = A0 (SoftAP)

    #Local Network Setup
    SSID = "ssid"                   #SSID of AP
    PSWD = "password"               #Password of AP
    SEC="4"                         #Security type of AP: 0-Open, 1-WEP, 2-WPA, 3-WPA2-AES, 4-WPA2-Mixed
    DHCP="1"                        #Get IP address for DHCP, 0-No, 1-Yes

    #Socket/Protocol Setup
    SOCKET = "0"                    #Select Socket 0-3
    PROTOCOL="4"                    #TCP=0, UDP=1, UDP-Lite=2 TCP-SSL=3(Hercules doesn't support), 4-MQTT

    #MQTT Setup
    PBTOPIC="$aws/things/<Thing_name>/shadow/update"  #Publish topic
    SBTOPIC="$aws/things/<Thing_name>/shadow/update/accepted"  #Subscribe topic
    MQTTMODE = "2"                  #Mode 0-None, 1-User Name/Password, 2-Certificates(AWS IoT)

    #TCP Keep-Alive
    TTI = 3000                      #Time-To-Idle (ms)

    #AWS IoT
    deviceID = "<AWS_ID>"                                   #AWS Device Id
    remoteURL = deviceID + ".iot.us-east-1.amazonaws.com"   #AWS Location URL

    #Client Mode
    useDNS = 1
    remoteIP = "192.168.1.48"       #Remote Server IP address (use 192.168.10.100 with AD Direct Mode)
    remotePort = "8883"             #Remote TCP port
    #---------------------------------------------------------------------------
    SSID = "debugAP"                  #SSID of AP
    PSWD = "!debugAP#"   #Password of AP
    SEC="4"                         #Security type of AP: 0-Open, 1-WEP, 2-WPA, 3-WPA2-AES, 4-WPA2-Mixed
    DHCP="1"                        #Get IP address for DHCP, 0-No, 1-Yes

    #Socket/Protocol Setup
    SOCKET = "0"                    #Select Socket 0-3
    PROTOCOL="4"                    #TCP=0, UDP=1, UDP-Lite=2 TCP-SSL=3(Hercules doesn't support), 4-MQTT-AWS

    #MQTT Setup

    PBTOPIC="$aws/things/lightbyulb/shadow/update"  #Publish topic
    SBTOPIC="$aws/things/lightbyulb/shadow/update/accepted"  #Subscribe topic
    #PBTOPIC="$aws/things/ISM_Theromstat/shadow/update"  #Publish topic
    #SBTOPIC="$aws/things/ISM_Theromstat/shadow/update/accepted"  #Subscribe topic
    MQTTMODE = "2"                  #Mode 0-None, 1-User Name/Password, 2-Certificates

    #TCP Keep-Alive
    TTI = 3000                      #Time-To-Idle (ms)

    #Client Mode
    useDNS = 1
    deviceID = "A19C8J2QVVZPKM"
    remoteURL = deviceID + ".iot.us-east-1.amazonaws.com"
    remoteIP = "192.168.1.48"       #Remote Server IP address (use 192.168.10.100 with AD Direct Mode)
    remotePort = "8883"             #Remote TCP port

    #Setup Thermosat------------------------------------------------------------
    mCnt = 10000                #Monitoring Count
    cal = 50                    #Temperature Calibration
    dCnt = 15                   #Delay seconds

    setPointF = 680.0                       #Set point deg F
    setPoint = (setPointF - 320) * (1/1.8)  #Set point deg C
    setLow = setPoint - 11.0                #Low event point
    setHigh = setPoint + 11.0               #High event point
    limitLow = setPoint - 44.0              #Low alarm limit
    limitHigh = setPoint + 44.0             #High alarm point

    alarmCount = 0              #Alarm counter
    dataCount = 0               #Data counter
    eventCount = 0              #Event counter
    locatCount = 0              #Location counter
    monitorCount = 0            #Monitor counter
    pointCount = 0              #Set point counter

    useSN = 0                   #0 = MAC, 1 = Serial Number
    useRC = 1                   #Return Channel, 0-No, 1-Yes
    #---------------------------------------------------------------------------

    minTmp = 999                #Minimum temperature
    maxTmp = -999               #Maximum temperature
    aveTmp = 0                  #Average temperature
    totalTmp = 0                #Total temperature

    deg = unichr(0xB0)          #Degree character
    ctrlQ = unichr(0x11)        #Ctrl-Q character
    exitApp = 0

    Button0 = '1'               #Up button
    Button1 = '1'               #Down button

    sn = ""                     #Serial Number

    #Start Application
    print( "Thermostat Demo:" )
    print( "Reporting cycle:        " + str(dCnt) + " seconds" )
    print( "Temperature Limit High: " + ("%.0F" % limitHigh) + " (" + ("%.1F" % (limitHigh/10)) + deg +"C/" + ("%.1F" % ((limitHigh*0.18)+32)) + deg + "F)" )
    print( "Temperature Set High:   " + ("%.0F" % setHigh) + " (" + ("%.1F" % (setHigh/10)) + deg +"C/" + ("%.1F" % ((setHigh*0.18)+32)) + deg + "F)" )
    print( "Temperature Set Point:  " + ("%.0F" % setPoint) + " (" + ("%.1F" % (setPoint/10)) + deg +"C/" + ("%.1F" % ((setPoint*0.18)+32)) + deg + "F)" )
    print( "Temperature Set Low:    " + ("%.0F" % setLow) + " (" + ("%.1F" % (setLow/10)) + deg +"C/" + ("%.1F" % ((setLow*0.18)+32)) + deg + "F)" )
    print( "Temperature Limit Low:  " + ("%.0F" % limitLow) + " (" + ("%.1F" % (limitLow/10)) + deg +"C/" + ("%.1F" % ((limitLow*0.18)+32)) + deg + "F)" )
    print( "Cycle Limit:            " + str(mCnt) )
    print( "Communications:         " + ser.portstr + "@" + str(baud))
    print( "" )

    #Additional features
    rcv = sendCMDBlocking(ser, "MT=1")                          #No Usage message on Error

    #Setup Read timeout
    rcv = sendCMDBlocking(ser, "P0="+SOCKET)                    #Select socket
    if (PROTOCOL == "3"):
        rcv = sendCMDBlocking(ser, "R2=500")                    #Read timeout(max polling rate) ms
    else:
        rcv = sendCMDBlocking(ser, "R2=10")                     #Read timeout(max polling rate) ms
    #End if
    rcv = sendCMDBlocking(ser, "R?")                            #Display Rx info

    #Setup Send timeout
    rcv = sendCMDBlocking(ser, "S2=250")                        #Send timeout ms

    #Join Network
    if (useSoftAP == 0):
        rcv = sendCMDBlocking(ser, "C1=" + SSID)                #Set SSID
        rcv = sendCMDBlocking(ser, "C2=" + PSWD)                #Set Password
        rcv = sendCMDBlocking(ser, "C3=4")                      #WPA-2 Mixed
        rcv = sendCMDBlocking(ser, "C4=1")                      #Use DHCP

        #Get start time
        st = time.time()

        #Join
        rcv = sendCMDBlocking(ser, "CD")                        #Leave network
        rcv = sendCMDBlocking(ser, "C0")                        #Join network
        if (rcv != '-1'):
            if "ERROR" in rcv:                                  #Check for error
                print( "Join failed to connect to network")
                sys.exit(-1)
            else:
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Joining: " + rcv[10:] )
            #End if
        else:
            print( "Joined timed out" )
            sys.exit(-1)
        #End if
    else:
        #SoftAP
        rcv = sendCMDBlocking(ser, "Z5")                        #Get MAC address
        print( "Starting Configuration Access Point, look for Thermostat_" + rcv )

        #Get start time
        st = time.time()

        #Setup SoftAP
        rcv = sendCMDBlocking(ser, "AS=1,Thermostat")           #Set SoftAP SSID

        #Start and wait for Join message
        rcv = sendCMDWait(ser, "A0","[JOIN   ]",300)            #Start SoftAP
        if (rcv != '-1'):                                       #Check for timeout
            if "ERROR" in rcv:                                  #Check for error
                print( "Join failed to connect to network")
                sys.exit(-1)
            else:
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Joining: " + rcv[10:] )
            #End if
        else:
            print( "Timeout, failed to connect to a network")
            serPort.write(ctrlQ)                                #Send Quit
            sys.exit(-1)
        #End if
    #End if

    #Serial Number
    if (useSN == 1):
        #Use Serial Number
        sn = sendCMDBlocking(ser, "ZS")         #Get Serial Number
    else:
        #Use MAC as serial number
        sn = sendCMDBlocking(ser, "Z5")         #Get MAC
        sn = sn.replace(':','')                 #Remove ':'
    #Endif

    #Configure LEDs
    rcv = sendCMDBlocking(ser, "G4=3,1")        #Red LED
    rcv = sendCMDBlocking(ser, "G4=4,1")        #Green LED

    #Green - OFF, RED - OFF
    rcv = sendCMDBlocking(ser, "G3=0,1,0")      #Red LED
    rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

    #Connect to server
    #Setup and Start Client
    rcv = sendCMDBlocking(ser, "P0="+SOCKET)                    #Select socket
    rcv = sendCMDBlocking(ser, "P1="+PROTOCOL)                  #TCP/TCP-SSL
    if (PROTOCOL == "4"):
        rcv = sendCMDBlocking(ser, "PM=0,"+PBTOPIC)             #Publish topic
        rcv = sendCMDBlocking(ser, "PM=1,"+SBTOPIC)             #Subscribe topic
        rcv = sendCMDBlocking(ser, "PM=2,"+MQTTMODE)            #MQTT Mode
    #Endif

    if ( useDNS == 0 ):
        rcv = sendCMDBlocking(ser, "P3="+remoteIP)              #Remote IP
    else:
        rcv = sendCMDBlocking(ser, "D0="+remoteURL)             #Remote URL
    #Endif

    rcv = sendCMDBlocking(ser, "P4="+remotePort)                #Remote Port

    if ((PROTOCOL == "0") or (PROTOCOL == "3")):
        rcv = sendCMDBlocking(ser, "PK=1," + str(TTI))          #TCP Keep-Alive(ms)
    #Endif

    rcv = sendCMDBlocking(ser, "P6=1")                          #Start Client
    if (rcv == '-1'):                                           #Check for timeout
        if "ERROR" in rcv:                                      #Check for error
            print( "Failed to connect to server")
            sys.exit(-1)
        #End if
    #End if

    #aws_json_did = "{\"did\":\""+ str(sn) + "\","
    aws_json_pre = "{\"state\":{\"reported\":"
    aws_json_pre_desired = "{\"state\":{\"desired\":"
    aws_json_post = "}}"

    #Send Set Point to server
    print( "Time: " + ("%012.3F" % (time.time()-st)) + " Set Point:   " + ("%.0F" % setPoint) + " (" + ("%.1F" % (setPoint/10.0)) + deg +"C/" + ("%.1F" % ((setPoint*0.18)+32)) + deg + "F)" )
    setPointF =((setPoint*0.18)+32)
    msg = aws_json_pre_desired
    msg += "{\"setPoint\":"+ ("%.0F" % setPointF) + "}"
    msg += aws_json_post
    data = bytearray("S3=" + str(len(msg)) + "\r" + str(msg))
    rcv0 = sendDataBlocking(ser, data)          #Send data
    if (rcv0 == '-1'):
        sys.exit(-1)
    #Endif
    pointCount += 1

    ser.flush
    #Reporting Loop
    while monitorCount < mCnt:
        #Clear buffer?
        rcv1 = bytearray( sendCMDBlocking(ser, "R0") )  #Read data

        #Get Thermistor value
        threm = sendCMDBlocking(ser, "G2=0,5")      #Read ADC0
        if (threm == 0):
            threm = sendCMDBlocking(ser, "G2=0,5")      #Read ADC0

        #Calculate temperature
        thremo_res =  43000.0 / (( 4096.0 / float(threm)) - 1 )
        logval = math.log( thremo_res / 10000.0 )
        tmp = 1.0 / ( logval / 3380.0 + 1.0 /298.15 ) -273.15

        #Calibrate temperature
        calTmp = (tmp * 10) - cal

        #Send Temperature to server
        print( "Time: " + ("%012.3F" % (time.time()-st)) + " Temperature: " + ("%.0F" % calTmp) + " (" + ("%.1F" % (calTmp/10)) + deg + "C/" + ("%.1F" % ((calTmp*0.18)+32)) + deg + "F)"  + " Cnt: " + str(monitorCount+1))
        calTmpF =((calTmp*0.18)+32)
        msg = aws_json_pre
        msg += "{\"intTemp\":"+ ("%.0F" % calTmpF) + "}"
        msg += aws_json_post
        data = bytearray("S3=" + str(len(msg)) + "\r" + str(msg))
        rcv0 = sendDataBlocking(ser, data)          #Send data
        if (rcv0 == '-1'):
            print("Exit intTemp")
            break
        #Endif
        dataCount += 1


        #Stats
        if (calTmp < minTmp):
            minTmp = calTmp             #Set new minimum
        if (calTmp > maxTmp):
            maxTmp = calTmp             #Set new maximum
        totalTmp += calTmp              #Sum temperature


        #Alarm Check
        if (calTmp + 0.5 >= limitHigh):
            print( "Time: " + ("%012.3F" % (time.time()-st)) + " Alarm, Limit High reached" )
            alarmCount += 1
        elif (calTmp - 0.5 <= limitLow):
            print( "Time: " + ("%012.3F" % (time.time()-st)) + " Alarm, Limit Low reached" )
            alarmCount += 1
        #Endif


        #Events
        if (calTmp + 0.5 >= setHigh):
            #Green - OFF, RED - ON
            rcv = sendCMDBlocking(ser, "G3=0,1,1")      #Red LED
            rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

            print( "Time: " + ("%012.3F" % (time.time()-st)) + " Event, Set High reached" )
            eventCount += 1
        elif (calTmp - 0.5 <= setLow):
            #Green - OFF, RED - ON
            rcv = sendCMDBlocking(ser, "G3=0,1,1")      #Red LED
            rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

            print( "Time: " + ("%012.3F" % (time.time()-st)) + " Event, Set Low reached" )
            eventCount += 1
        else:
            #Green - ON, RED - OFF
            rcv = sendCMDBlocking(ser, "G3=0,1,0")      #Red LED
            rcv = sendCMDBlocking(ser, "G3=1,1,1")      #Green LED
        #Endif


        #Loop Delay
        for j in range(1,((dCnt-2)*8)):
            time.sleep(.125)                             #125ms

            #Button Events
            Button0 = sendCMDBlocking(ser, "G2=0,3")    #Set point up (GPIO0)
            Button1 = sendCMDBlocking(ser, "G2=1,3")    #Set point down (GPIO1)

            #Press both buttons to exit
            if ( ('0' in Button0) and ('0' in Button1) ) and (exitApp != 1):
                exitApp = 1
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " User request, exiting" )
                Button0 = '1'                           #Clear button
                Button1 = '1'                           #Clear button
                cycles = monitorCount + 1               #Save count
                monitorCount = mCnt                     #Move count to limit
            #End if

            #Check Up Button
            if ('0' in Button0) and (exitApp != 1):
                #increment set point
                setPoint += 5.5556
                setPointF = ((setPoint/10) * 1.8) + 32

                #Send event to server
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Event, Set Point, UP " + ("%.1F" % (setPointF)) )
                eventCount += 1
            #Endif

            #Check Down button
            if ('0' in Button1) and (exitApp != 1):
                #decrement set point
                setPoint -= 5.5556
                setPointF = ((setPoint/10) * 1.8) + 32

                #Send event to server
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Event, Set Point, DOWN "  + ("%.1F" % (setPointF)) )
                eventCount += 1
            #Endif

            #Update setting if either button pressed
            if ( ('0' in Button0) or ('0' in Button1) and (exitApp != 1) ):
                #Calculate limits
                setLow = setPoint - 11.0
                setHigh = setPoint + 11.0
                limitLow = setPoint - 44.0
                limitHigh = setPoint + 44.0

                #Send Set Point to server
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Set Point:   " + ("%.0F" % setPoint) + " (" + ("%.1F" % (setPoint/10.0)) + deg + "C/" + ("%.1F" % ((setPoint*0.18)+32)) + deg + "F)" )
                msg = aws_json_pre_desired
                msg += "{\"enabled\":true,"
                msg += "\"setPoint\":" + ("%.0F" % setPointF)+ "}"
                msg += aws_json_post
                data = bytearray("S3=" + str(len(msg)) + "\r" + str(msg))
                rcv0 = sendDataBlocking(ser, data)          #Send data
                if (rcv0 == '-1'):
                    print("Exit button press")
                    break
                #Endif
                pointCount += 1
            #Endif

            if (useRC == 1):
                #Get server commands
                if (rcv1 == ""):
                    rcv1 = bytearray( sendCMDBlocking(ser, "R0") )  #Read data


                #For me? command format: "desired","command",data
                if ("desired" in rcv1):
                    #Process server commands
                    #print ("Received command: " + str(rcv1))
                    if ("setPoint" in rcv1):
                        rcv1=rcv1.replace("{","")
                        rcv1=rcv1.replace("}","")
                        rcv1=rcv1.replace(",",":")
                        #print("rcv1 = " + str(rcv1))
                        rd = (rcv1.split(":"))
                        rd = rd[rd.index("\"setPoint\"")+1]
                        print ("Received command: setPoint = " + rd)
                        setPointF=int(rd)
                        setPoint = (setPointF - 32) * (10/1.8)  #Set point deg C
                        setLow = setPoint - 11.0
                        setHigh = setPoint + 11.0

                        print( "Time: " + ("%012.3F" % (time.time()-st)) + " Set Point:   " + ("%.1F" % setPointF) + deg + "F" )

                        #Send confirmation back to server
                        msg = aws_json_pre
                        msg += "{\"setPoint\":"+ ("%.0F" % setPointF) + "}"
                        msg += aws_json_post
                        data = bytearray("S3=" + str(len(msg)) + "\r" + str(msg))
                        rcv0 = sendDataBlocking(ser, data)          #Send data
                        if (rcv0 == '-1'):
                            print("Exit setPoint cmd")
                            break
                        #Endif
                        eventCount += 1
                        break
                    #Endif


                #For me? command format: sn,command,option
                if (sn in rcv1):
                    #print ("Received command: " + str(rcv1))
                    #Process server commands
                    if ("setpoint" in rcv1):
                        setPointF = int(rcv1.split(",")[1])
                        setPoint = int((setPointF - 320) * (1/1.8))  #Set point deg C
                        setLow = setPoint - 11.0
                        setHigh = setPoint + 11.0
                        limitLow = setPoint - 44.0
                        limitHigh = setPoint + 44.0

                        print("setLow = " + str(setLow) )
                        print("setHigh = " + str(setHigh) )
                        print("limitLow = " + str(limitLow) )
                        print("limitHigh = " + str(limitHigh) )

                        print( "Time: " + ("%012.3F" % (time.time()-st)) + " Set Point:   " + ("%.0F" % setPoint) + " (" + ("%.1F" % (setPoint/10.0)) + deg + "C/" + ("%.1F" % ((setPoint*0.18)+32)) + deg + "F)" )

                        #Send confirmation back to server
                        eventCount += 1
                    #Endif

                    if ("pointUp" in rcv1):
                        setPoint += 5.5
                        setLow = setPoint - 11.0
                        setHigh = setPoint + 11.0
                        #print("setLow = " + str(setLow) )
                        #print("setHigh = " + str(setHigh) )
                        print( "Time: " + ("%012.3F" % (time.time()-st)) + " Set Point:   " + ("%.0F" % setPoint) + " (" + ("%.1F" % (setPoint/10.0)) + deg + "C/" + ("%.1F" % ((setPoint*0.18)+32)) + deg + "F)" )
                        #Send confirmation back to server
                        eventCount += 1
                    #Endif

                    if ("pointDown" in rcv1):
                        setPoint -= 5.5
                        setLow = setPoint - 11.0
                        setHigh = setPoint + 11.0
                        #print("setLow = " + str(setLow) )
                        #print("setHigh = " + str(setHigh) )
                        print( "Time: " + ("%012.3F" % (time.time()-st)) + " Set Point:   " + ("%.0F" % setPoint) + " (" + ("%.1F" % (setPoint/10.0)) + deg + "C/" + ("%.1F" % ((setPoint*0.18)+32)) + deg + "F)" )
                        #Send confirmation back to server
                        eventCount += 1
                    #Endif

                    if ("shutdown" in rcv1):
                        print("Exit shutdown")
                        exitApp = 1
                        cycles = monitorCount + 1               #Save count
                        monitorCount = mCnt                     #Move count to limit
                        break
                    #Endif
                #Endif-sn
             #Endif-useRC
            rcv1=""
        #End for

        monitorCount += 1                       #Monitor counter
    #End while

    eventCount += 1

    #Leave Network
    rcv = sendCMDBlocking(ser, "P6=0")          #Stop Client
    rcv = sendCMDBlocking(ser, "CD")            #Leave network

    #Green - OFF, RED - OFF
    rcv = sendCMDBlocking(ser, "G3=0,1,0")      #Red LED
    rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

    #Close serial port
    ser.close()


    #Statistics
    aveTmp = totalTmp / dataCount
    print( "" )
    print( "Summary:" )
    print( "Minimum Temperature: " + ("%.0F" % minTmp) + " (" + ("%.1F" % (minTmp/10)) + deg + "C/" + ("%.1F" % ((minTmp*0.18)+32)) + deg + "F)" )
    print( "Average Temperature: " + ("%.0F" % aveTmp) + " (" + ("%.1F" % (aveTmp/10)) + deg + "C/" + ("%.1F" % ((aveTmp*0.18)+32)) + deg + "F)" )
    print( "Maximum Temperature: " + ("%.0F" % maxTmp) + " (" + ("%.1F" % (maxTmp/10)) + deg + "C/" + ("%.1F" % ((maxTmp*0.18)+32)) + deg + "F)" )
    print( "" )
    print( "Alarm Count:         " + str(alarmCount) )
    print( "Data Count:          " + str(dataCount + pointCount) )
    print( "Event Count:         " + str(eventCount) )
    print( "Location Count:      " + str(locatCount) )
    print( "" )
    if (exitApp == 0):
        print( "Total Cycles:        " + str(monitorCount) )
    else:
        print( "Total Cycles:        " + str(cycles) )

if __name__ == '__main__':
    sys.exit(main())

